@@ -24,13 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
-#include <stdio.h>
+#include "config.h"
 
 #include "gdktypes.h"
 #include "gdkprivate-x11.h"
 
+#include <stdio.h>
+
+
 gboolean          _gdk_use_xshm = TRUE;  /* used as a cmd line arg */
-Display          *gdk_display = NULL;
 GdkAtom           _gdk_selection_property;
 gboolean          _gdk_synchronize = FALSE;
