@@ -21,12 +21,12 @@
 #include "config.h"
 #include <string.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
 #include "gtkfilefilter.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
 
-#include "gtkalias.h"
-
 typedef struct _GtkFileFilterClass GtkFileFilterClass;
 typedef struct _FilterRule FilterRule;
 
@@ -43,13 +43,13 @@ typedef enum {
 
 struct _GtkFileFilterClass
 {
-  GtkObjectClass parent_class;
+  GInitiallyUnownedClass parent_class;
 };
 
 struct _GtkFileFilter
 {
-  GtkObject parent_instance;
-  
+  GInitiallyUnowned parent_instance;
+
   gchar *name;
   GSList *rules;
 
@@ -76,7 +76,7 @@ struct _FilterRule
 static void gtk_file_filter_finalize   (GObject            *object);
 
 
-G_DEFINE_TYPE (GtkFileFilter, gtk_file_filter, GTK_TYPE_OBJECT)
+G_DEFINE_TYPE (GtkFileFilter, gtk_file_filter, G_TYPE_INITIALLY_UNOWNED)
 
 static void
 gtk_file_filter_init (GtkFileFilter *object)
@@ -421,6 +421,3 @@ gtk_file_filter_filter (GtkFileFilter           *filter,
 
   return FALSE;
 }
-
-#define __GTK_FILE_FILTER_C__
-#include "gtkaliasdef.c"
