@@ -25,16 +25,20 @@
  */
 
 #include "config.h"
+
+#include "gtkmenubar.h"
+
 #include "gdk/gdkkeysyms.h"
 #include "gtkbindings.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtkmenubar.h"
 #include "gtkmenuitem.h"
 #include "gtkmenuprivate.h"
 #include "gtksettings.h"
-#include "gtkintl.h"
+#include "gtksizerequest.h"
 #include "gtkwindow.h"
+
+#include "gtkintl.h"
 #include "gtkprivate.h"
 
 #define BORDER_SPACING  0
@@ -64,12 +68,16 @@ static void gtk_menu_bar_get_property      (GObject             *object,
 					    GParamSpec          *pspec);
 static void gtk_menu_bar_size_request      (GtkWidget       *widget,
 					    GtkRequisition  *requisition);
+static void gtk_menu_bar_get_preferred_width (GtkWidget     *widget,
+					      gint          *minimum,
+					      gint          *natural);
+static void gtk_menu_bar_get_preferred_height (GtkWidget    *widget,
+					       gint         *minimum,
+					       gint         *natural);
 static void gtk_menu_bar_size_allocate     (GtkWidget       *widget,
 					    GtkAllocation   *allocation);
-static void gtk_menu_bar_paint             (GtkWidget       *widget,
-					    GdkRectangle    *area);
-static gint gtk_menu_bar_expose            (GtkWidget       *widget,
-					    GdkEventExpose  *event);
+static gint gtk_menu_bar_draw              (GtkWidget       *widget,
+                                            cairo_t         *cr);
 static void gtk_menu_bar_hierarchy_changed (GtkWidget       *widget,
 					    GtkWidget       *old_toplevel);
 static gint gtk_menu_bar_get_popup_delay   (GtkMenuShell    *menu_shell);
@@ -96,9 +104,10 @@ gtk_menu_bar_class_init (GtkMenuBarClass *class)
   gobject_class->get_property = gtk_menu_bar_get_property;
   gobject_class->set_property = gtk_menu_bar_set_property;
 
-  widget_class->size_request = gtk_menu_bar_size_request;
+  widget_class->get_preferred_width = gtk_menu_bar_get_preferred_width;
+  widget_class->get_preferred_height = gtk_menu_bar_get_preferred_height;
   widget_class->size_allocate = gtk_menu_bar_size_allocate;
-  widget_class->expose_event = gtk_menu_bar_expose;
+  widget_class->draw = gtk_menu_bar_draw;
   widget_class->hierarchy_changed = gtk_menu_bar_hierarchy_changed;
   
   menu_shell_class->submenu_placement = GTK_TOP_BOTTOM;
@@ -199,14 +208,6 @@ gtk_menu_bar_class_init (GtkMenuBarClass *class)
                                                              DEFAULT_IPADDING,
                                                              GTK_PARAM_READABLE));
 
-  gtk_settings_install_property (g_param_spec_int ("gtk-menu-bar-popup-delay",
-						   P_("Delay before drop down menus appear"),
-						   P_("Delay before the submenus of a menu bar appear"),
-						   0,
-						   G_MAXINT,
-						   0,
-						   GTK_PARAM_READWRITE));
-
   g_type_class_add_private (gobject_class, sizeof (GtkMenuBarPrivate));
 }
 
@@ -307,7 +308,7 @@ gtk_menu_bar_size_request (GtkWidget      *widget,
               gint toggle_size;
 
 	      GTK_MENU_ITEM (child)->show_submenu_indicator = FALSE;
-	      gtk_widget_size_request (child, &child_requisition);
+              gtk_widget_get_preferred_size (child, &child_requisition, NULL);
               gtk_menu_item_toggle_size_request (GTK_MENU_ITEM (child),
                                                  &toggle_size);
 
@@ -353,6 +354,30 @@ gtk_menu_bar_size_request (GtkWidget      *widget,
     }
 }
 
+static void
+gtk_menu_bar_get_preferred_width (GtkWidget *widget,
+				  gint      *minimum,
+				  gint      *natural)
+{
+  GtkRequisition requisition;
+
+  gtk_menu_bar_size_request (widget, &requisition);
+
+  *minimum = *natural = requisition.width;
+}
+
+static void
+gtk_menu_bar_get_preferred_height (GtkWidget *widget,
+				   gint      *minimum,
+				   gint      *natural)
+{
+  GtkRequisition requisition;
+
+  gtk_menu_bar_size_request (widget, &requisition);
+
+  *minimum = *natural = requisition.height;
+}
+
 static void
 gtk_menu_bar_size_allocate (GtkWidget     *widget,
 			    GtkAllocation *allocation)
@@ -424,8 +449,8 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
 	      
 	      gtk_menu_item_toggle_size_request (GTK_MENU_ITEM (child),
 						 &toggle_size);
-	      gtk_widget_get_child_requisition (child, &child_requisition);
-	    
+              gtk_widget_get_preferred_size (child, &child_requisition, NULL);
+
 	      if (priv->child_pack_direction == GTK_PACK_DIRECTION_LTR ||
 		  priv->child_pack_direction == GTK_PACK_DIRECTION_RTL)
 		child_requisition.width += toggle_size;
@@ -474,8 +499,8 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
 	      
 	      gtk_menu_item_toggle_size_request (GTK_MENU_ITEM (child),
 						 &toggle_size);
-	      gtk_widget_get_child_requisition (child, &child_requisition);
-	      
+              gtk_widget_get_preferred_size (child, &child_requisition, NULL);
+
 	      if (priv->child_pack_direction == GTK_PACK_DIRECTION_LTR ||
 		  priv->child_pack_direction == GTK_PACK_DIRECTION_RTL)
 		child_requisition.width += toggle_size;
@@ -510,44 +535,24 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
     }
 }
 
-static void
-gtk_menu_bar_paint (GtkWidget    *widget,
-                    GdkRectangle *area)
-{
-  g_return_if_fail (GTK_IS_MENU_BAR (widget));
-
-  if (gtk_widget_is_drawable (widget))
-    {
-      GtkAllocation allocation;
-      guint border;
-
-      border = gtk_container_get_border_width (GTK_CONTAINER (widget));
-      gtk_widget_get_allocation (widget, &allocation);
-
-      gtk_paint_box (gtk_widget_get_style (widget),
-                     gtk_widget_get_window (widget),
-                     gtk_widget_get_state (widget),
-                     get_shadow_type (GTK_MENU_BAR (widget)),
-		     area, widget, "menubar",
-		     border, border,
-                     allocation.width - border * 2,
-                     allocation.height - border * 2);
-    }
-}
-
 static gint
-gtk_menu_bar_expose (GtkWidget      *widget,
-		     GdkEventExpose *event)
+gtk_menu_bar_draw (GtkWidget *widget,
+		   cairo_t   *cr)
 {
-  g_return_val_if_fail (GTK_IS_MENU_BAR (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
+  int border;
 
-  if (gtk_widget_is_drawable (widget))
-    {
-      gtk_menu_bar_paint (widget, &event->area);
+  border = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
-      GTK_WIDGET_CLASS (gtk_menu_bar_parent_class)->expose_event (widget, event);
-    }
+  gtk_paint_box (gtk_widget_get_style (widget),
+                 cr,
+                 gtk_widget_get_state (widget),
+                 get_shadow_type (GTK_MENU_BAR (widget)),
+                 widget, "menubar",
+                 border, border,
+                 gtk_widget_get_allocated_width (widget) - border * 2,
+                 gtk_widget_get_allocated_height (widget) - border * 2);
+
+  GTK_WIDGET_CLASS (gtk_menu_bar_parent_class)->draw (widget, cr);
 
   return FALSE;
 }
