@@ -25,9 +25,12 @@
  */
 
 #include "config.h"
+
+#include "gtkradiobutton.h"
+
+#include "gtkbuttonprivate.h"
 #include "gtklabel.h"
 #include "gtkmarshalers.h"
-#include "gtkradiobutton.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
@@ -73,7 +76,7 @@
  *
  *    GtkWidget *window, *radio1, *radio2, *box, *entry;
  *    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
- *    box = gtk_vbox_new (TRUE, 2);
+ *    box = gtk_box_new (GTK_ORIENTATION_VERTICAL, TRUE, 2);
  *
  *    /&ast; Create a radio button with a GtkEntry widget &ast;/
  *    radio1 = gtk_radio_button_new (NULL);
@@ -103,7 +106,7 @@
  */
 
 
-struct _GtkRadioButtonPriv
+struct _GtkRadioButtonPrivate
 {
   GSList *group;
 };
@@ -114,12 +117,12 @@ enum {
 };
 
 
-static void     gtk_radio_button_destroy        (GtkObject           *object);
+static void     gtk_radio_button_destroy        (GtkWidget           *widget);
 static gboolean gtk_radio_button_focus          (GtkWidget           *widget,
 						 GtkDirectionType     direction);
 static void     gtk_radio_button_clicked        (GtkButton           *button);
 static void     gtk_radio_button_draw_indicator (GtkCheckButton      *check_button,
-						 GdkRectangle        *area);
+						 cairo_t             *cr);
 static void     gtk_radio_button_set_property   (GObject             *object,
 						 guint                prop_id,
 						 const GValue        *value,
@@ -137,13 +140,11 @@ static void
 gtk_radio_button_class_init (GtkRadioButtonClass *class)
 {
   GObjectClass *gobject_class;
-  GtkObjectClass *object_class;
   GtkButtonClass *button_class;
   GtkCheckButtonClass *check_button_class;
   GtkWidgetClass *widget_class;
 
   gobject_class = G_OBJECT_CLASS (class);
-  object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   button_class = (GtkButtonClass*) class;
   check_button_class = (GtkCheckButtonClass*) class;
@@ -163,8 +164,7 @@ gtk_radio_button_class_init (GtkRadioButtonClass *class)
 							P_("The radio button whose group this widget belongs to."),
 							GTK_TYPE_RADIO_BUTTON,
 							GTK_PARAM_WRITABLE));
-  object_class->destroy = gtk_radio_button_destroy;
-
+  widget_class->destroy = gtk_radio_button_destroy;
   widget_class->focus = gtk_radio_button_focus;
 
   button_class->clicked = gtk_radio_button_clicked;
@@ -187,32 +187,32 @@ gtk_radio_button_class_init (GtkRadioButtonClass *class)
    * Since: 2.4
    */
   group_changed_signal = g_signal_new (I_("group-changed"),
-				       G_OBJECT_CLASS_TYPE (object_class),
+				       G_OBJECT_CLASS_TYPE (gobject_class),
 				       G_SIGNAL_RUN_FIRST,
 				       G_STRUCT_OFFSET (GtkRadioButtonClass, group_changed),
 				       NULL, NULL,
 				       _gtk_marshal_VOID__VOID,
 				       G_TYPE_NONE, 0);
 
-  g_type_class_add_private (class, sizeof (GtkRadioButtonPriv));
+  g_type_class_add_private (class, sizeof (GtkRadioButtonPrivate));
 }
 
 static void
 gtk_radio_button_init (GtkRadioButton *radio_button)
 {
-  GtkRadioButtonPriv *priv;
+  GtkRadioButtonPrivate *priv;
 
   radio_button->priv = G_TYPE_INSTANCE_GET_PRIVATE (radio_button,
                                                     GTK_TYPE_RADIO_BUTTON,
-                                                    GtkRadioButtonPriv);
+                                                    GtkRadioButtonPrivate);
   priv = radio_button->priv;
 
   gtk_widget_set_has_window (GTK_WIDGET (radio_button), FALSE);
   gtk_widget_set_receives_default (GTK_WIDGET (radio_button), FALSE);
 
-  GTK_TOGGLE_BUTTON (radio_button)->active = TRUE;
+  _gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), TRUE);
 
-  GTK_BUTTON (radio_button)->depress_on_activate = FALSE;
+  GTK_BUTTON (radio_button)->priv->depress_on_activate = FALSE;
 
   priv->group = g_slist_prepend (NULL, radio_button);
 
@@ -267,8 +267,8 @@ gtk_radio_button_get_property (GObject    *object,
 /**
  * gtk_radio_button_set_group:
  * @radio_button: a #GtkRadioButton.
- * @group: an existing radio button group, such as one returned from
- *  gtk_radio_button_get_group().
+ * @group: (transfer none) (element-type GtkRadioButton): an existing radio
+ *     button group, such as one returned from gtk_radio_button_get_group().
  *
  * Sets a #GtkRadioButton's group. It should be noted that this does not change
  * the layout of your interface in any way, so if you are changing the group,
@@ -279,7 +279,7 @@ void
 gtk_radio_button_set_group (GtkRadioButton *radio_button,
 			    GSList         *group)
 {
-  GtkRadioButtonPriv *priv;
+  GtkRadioButtonPrivate *priv;
   GtkWidget *old_group_singleton = NULL;
   GtkWidget *new_group_singleton = NULL;
 
@@ -346,14 +346,70 @@ gtk_radio_button_set_group (GtkRadioButton *radio_button,
   g_object_unref (radio_button);
 }
 
+/**
+ * gtk_radio_button_join_group:
+ * @radio_button: the #GtkRadioButton object
+ * @group_source: (allow-none): a radio button object whos group we are 
+ *   joining, or %NULL to remove the radio button from its group
+ *
+ * Joins a #GtkRadioButton object to the group of another #GtkRadioButton object
+ *
+ * Use this in language bindings instead of the gtk_radio_button_get_group() 
+ * and gtk_radio_button_set_group() methods
+ *
+ * A common way to set up a group of radio buttons is the following:
+ * |[
+ *   GtkRadioButton *radio_button;
+ *   GtkRadioButton *last_button;
+ *
+ *   while (/&ast; more buttons to add &ast;/)
+ *     {
+ *        radio_button = gtk_radio_button_new (...);
+ *
+ *        gtk_radio_button_join_group (radio_button, last_button);
+ *        last_button = radio_button;
+ *     }
+ * ]|
+ *
+ * Since: 3.0
+ */
+void
+gtk_radio_button_join_group (GtkRadioButton *radio_button, 
+			     GtkRadioButton *group_source)
+{
+  g_return_if_fail (GTK_IS_RADIO_BUTTON (radio_button));
+  g_return_if_fail (group_source == NULL || GTK_IS_RADIO_BUTTON (group_source));
+
+  if (group_source)
+    {
+      GSList *group;
+      group = gtk_radio_button_get_group (group_source);
+
+      if (!group)
+        {
+          /* if we are not already part of a group we need to set up a new one
+             and then get the newly created group */
+          gtk_radio_button_set_group (group_source, NULL);
+          group = gtk_radio_button_get_group (group_source);
+        }
+
+      gtk_radio_button_set_group (radio_button, group);
+    }
+  else
+    {
+      gtk_radio_button_set_group (radio_button, NULL);
+    }
+}
+
 /**
  * gtk_radio_button_new:
- * @group: an existing radio button group, or %NULL if you are creating a new group.
+ * @group: (allow-none): an existing radio button group, or %NULL if you are
+ *  creating a new group.
  *
  * Creates a new #GtkRadioButton. To be of any practical value, a widget should
  * then be packed into the radio button.
  *
- * Returns: a new radio button.
+ * Returns: a new radio button
  */
 GtkWidget*
 gtk_radio_button_new (GSList *group)
@@ -370,13 +426,13 @@ gtk_radio_button_new (GSList *group)
 
 /**
  * gtk_radio_button_new_with_label:
- * @group: an existing radio button group, or %NULL if you are creating a new
- *  group.
+ * @group: (allow-none): an existing radio button group, or %NULL if you are
+ *  creating a new group.
  * @label: the text label to display next to the radio button.
  *
  * Creates a new #GtkRadioButton with a text label.
  *
- * Returns: a new radio button.
+ * Returns: (transfer full): a new radio button.
  */
 GtkWidget*
 gtk_radio_button_new_with_label (GSList      *group,
@@ -395,16 +451,17 @@ gtk_radio_button_new_with_label (GSList      *group,
 
 /**
  * gtk_radio_button_new_with_mnemonic:
- * @group: the radio button group
+ * @group: (allow-none): the radio button group
  * @label: the text of the button, with an underscore in front of the
  *         mnemonic character
- * @returns: a new #GtkRadioButton
  *
- * Creates a new #GtkRadioButton containing a label, adding it to the same 
- * group as @group. The label will be created using 
- * gtk_label_new_with_mnemonic(), so underscores in @label indicate the 
+ * Creates a new #GtkRadioButton containing a label, adding it to the same
+ * group as @group. The label will be created using
+ * gtk_label_new_with_mnemonic(), so underscores in @label indicate the
  * mnemonic for the button.
- **/
+ *
+ * Returns: (transfer full): a new #GtkRadioButton
+ */
 GtkWidget*
 gtk_radio_button_new_with_mnemonic (GSList      *group,
 				    const gchar *label)
@@ -424,12 +481,13 @@ gtk_radio_button_new_with_mnemonic (GSList      *group,
 
 /**
  * gtk_radio_button_new_from_widget:
- * @radio_group_member: an existing #GtkRadioButton.
+ * @radio_group_member: (allow-none): an existing #GtkRadioButton.
  *
- * Creates a new #GtkRadioButton, adding it to the same group as @radio_group_member.
- * As with gtk_radio_button_new(), a widget should be packed into the radio button.
+ * Creates a new #GtkRadioButton, adding it to the same group as
+ * @radio_group_member. As with gtk_radio_button_new(), a widget
+ * should be packed into the radio button.
  *
- * Returns: a new radio button.
+ * Returns: (transfer full): a new radio button.
  */
 GtkWidget*
 gtk_radio_button_new_from_widget (GtkRadioButton *radio_group_member)
@@ -442,13 +500,13 @@ gtk_radio_button_new_from_widget (GtkRadioButton *radio_group_member)
 
 /**
  * gtk_radio_button_new_with_label_from_widget:
- * @radio_group_member: widget to get radio group from or %NULL
+ * @radio_group_member: (allow-none): widget to get radio group from or %NULL
  * @label: a text string to display next to the radio button.
  *
- * Creates a new #GtkRadioButton with a text label, adding it to the same group
- * as @radio_group_member.
+ * Creates a new #GtkRadioButton with a text label, adding it to
+ * the same group as @radio_group_member.
  *
- * Returns: a new radio button.
+ * Returns: (transfer none): a new radio button.
  */
 GtkWidget*
 gtk_radio_button_new_with_label_from_widget (GtkRadioButton *radio_group_member,
@@ -465,11 +523,12 @@ gtk_radio_button_new_with_label_from_widget (GtkRadioButton *radio_group_member,
  * @radio_group_member: (allow-none): widget to get radio group from or %NULL
  * @label: the text of the button, with an underscore in front of the
  *         mnemonic character
- * @returns: a new #GtkRadioButton
  *
  * Creates a new #GtkRadioButton containing a label. The label
  * will be created using gtk_label_new_with_mnemonic(), so underscores
  * in @label indicate the mnemonic for the button.
+ *
+ * Returns: (transfer full): a new #GtkRadioButton
  **/
 GtkWidget*
 gtk_radio_button_new_with_mnemonic_from_widget (GtkRadioButton *radio_group_member,
@@ -503,11 +562,11 @@ gtk_radio_button_get_group (GtkRadioButton *radio_button)
 
 
 static void
-gtk_radio_button_destroy (GtkObject *object)
+gtk_radio_button_destroy (GtkWidget *widget)
 {
   GtkWidget *old_group_singleton = NULL;
-  GtkRadioButton *radio_button = GTK_RADIO_BUTTON (object);
-  GtkRadioButtonPriv *priv = radio_button->priv;
+  GtkRadioButton *radio_button = GTK_RADIO_BUTTON (widget);
+  GtkRadioButtonPrivate *priv = radio_button->priv;
   GtkRadioButton *tmp_button;
   GSList *tmp_list;
   gboolean was_in_group;
@@ -536,7 +595,7 @@ gtk_radio_button_destroy (GtkObject *object)
   if (was_in_group)
     g_signal_emit (radio_button, group_changed_signal, 0);
 
-  GTK_OBJECT_CLASS (gtk_radio_button_parent_class)->destroy (object);
+  GTK_WIDGET_CLASS (gtk_radio_button_parent_class)->destroy (widget);
 }
 
 static void
@@ -545,9 +604,12 @@ get_coordinates (GtkWidget    *widget,
 		 gint         *x,
 		 gint         *y)
 {
-  *x = widget->allocation.x + widget->allocation.width / 2;
-  *y = widget->allocation.y + widget->allocation.height / 2;
-  
+  GtkAllocation allocation;
+
+  gtk_widget_get_allocation (widget, &allocation);
+  *x = allocation.x + allocation.width / 2;
+  *y = allocation.y + allocation.height / 2;
+
   gtk_widget_translate_coordinates (widget, reference, *x, *y, x, y);
 }
 
@@ -588,13 +650,13 @@ gtk_radio_button_focus (GtkWidget         *widget,
 			GtkDirectionType   direction)
 {
   GtkRadioButton *radio_button = GTK_RADIO_BUTTON (widget);
-  GtkRadioButtonPriv *priv = radio_button->priv;
+  GtkRadioButtonPrivate *priv = radio_button->priv;
   GSList *tmp_slist;
 
   /* Radio buttons with draw_indicator unset focus "normally", since
    * they look like buttons to the user.
    */
-  if (!GTK_TOGGLE_BUTTON (widget)->draw_indicator)
+  if (!gtk_toggle_button_get_mode (GTK_TOGGLE_BUTTON (widget)))
     return GTK_WIDGET_CLASS (gtk_radio_button_parent_class)->focus (widget, direction);
   
   if (gtk_widget_is_focus (widget))
@@ -708,7 +770,7 @@ gtk_radio_button_focus (GtkWidget         *widget,
       tmp_slist = priv->group;
       while (tmp_slist)
 	{
-	  if (GTK_TOGGLE_BUTTON (tmp_slist->data)->active)
+	  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp_slist->data)))
 	    selected_button = tmp_slist->data;
 	  tmp_slist = tmp_slist->next;
 	}
@@ -725,7 +787,7 @@ static void
 gtk_radio_button_clicked (GtkButton *button)
 {
   GtkRadioButton *radio_button = GTK_RADIO_BUTTON (button);
-  GtkRadioButtonPriv *priv = radio_button->priv;
+  GtkRadioButtonPrivate *priv = radio_button->priv;
   GtkToggleButton *toggle_button = GTK_TOGGLE_BUTTON (button);
   GtkToggleButton *tmp_button;
   GtkStateType new_state;
@@ -737,7 +799,7 @@ gtk_radio_button_clicked (GtkButton *button)
 
   g_object_ref (GTK_WIDGET (button));
 
-  if (toggle_button->active)
+  if (gtk_toggle_button_get_active (toggle_button))
     {
       tmp_button = NULL;
       tmp_list = priv->group;
@@ -747,7 +809,8 @@ gtk_radio_button_clicked (GtkButton *button)
 	  tmp_button = tmp_list->data;
 	  tmp_list = tmp_list->next;
 
-	  if (tmp_button->active && tmp_button != toggle_button)
+          if (tmp_button != toggle_button &&
+              gtk_toggle_button_get_active (tmp_button))
 	    break;
 
 	  tmp_button = NULL;
@@ -755,19 +818,21 @@ gtk_radio_button_clicked (GtkButton *button)
 
       if (!tmp_button)
 	{
-	  new_state = (button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_ACTIVE);
+	  new_state = (button->priv->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_ACTIVE);
 	}
       else
 	{
 	  toggled = TRUE;
-	  toggle_button->active = !toggle_button->active;
-	  new_state = (button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);
+          _gtk_toggle_button_set_active (toggle_button,
+                                         !gtk_toggle_button_get_active (toggle_button));
+	  new_state = (button->priv->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);
 	}
     }
   else
     {
       toggled = TRUE;
-      toggle_button->active = !toggle_button->active;
+      _gtk_toggle_button_set_active (toggle_button,
+                                     !gtk_toggle_button_get_active (toggle_button));
 
       tmp_list = priv->group;
       while (tmp_list)
@@ -775,22 +840,22 @@ gtk_radio_button_clicked (GtkButton *button)
 	  tmp_button = tmp_list->data;
 	  tmp_list = tmp_list->next;
 
-	  if (tmp_button->active && (tmp_button != toggle_button))
+	  if (gtk_toggle_button_get_active (tmp_button) && (tmp_button != toggle_button))
 	    {
 	      gtk_button_clicked (GTK_BUTTON (tmp_button));
 	      break;
 	    }
 	}
 
-      new_state = (button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_ACTIVE);
+      new_state = (button->priv->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_ACTIVE);
     }
 
-  if (toggle_button->inconsistent)
+  if (gtk_toggle_button_get_inconsistent (toggle_button))
     depressed = FALSE;
-  else if (button->in_button && button->button_down)
-    depressed = !toggle_button->active;
+  else if (button->priv->in_button && button->priv->button_down)
+    depressed = !gtk_toggle_button_get_active (toggle_button);
   else
-    depressed = toggle_button->active;
+    depressed = gtk_toggle_button_get_active (toggle_button);
 
   if (gtk_widget_get_state (GTK_WIDGET (button)) != new_state)
     gtk_widget_set_state (GTK_WIDGET (button), new_state);
@@ -811,14 +876,17 @@ gtk_radio_button_clicked (GtkButton *button)
 
 static void
 gtk_radio_button_draw_indicator (GtkCheckButton *check_button,
-				 GdkRectangle   *area)
+				 cairo_t        *cr)
 {
+  GtkAllocation allocation;
   GtkWidget *widget;
   GtkWidget *child;
   GtkButton *button;
   GtkToggleButton *toggle_button;
   GtkStateType state_type;
   GtkShadowType shadow_type;
+  GtkStyle *style;
+  GdkWindow *window;
   gint x, y;
   gint indicator_size, indicator_spacing;
   gint focus_width;
@@ -827,71 +895,63 @@ gtk_radio_button_draw_indicator (GtkCheckButton *check_button,
   gboolean interior_focus;
 
   widget = GTK_WIDGET (check_button);
+  button = GTK_BUTTON (check_button);
+  toggle_button = GTK_TOGGLE_BUTTON (check_button);
 
-  if (gtk_widget_is_drawable (widget))
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  style = gtk_widget_get_style (widget);
+  gtk_widget_style_get (widget,
+                        "interior-focus", &interior_focus,
+                        "focus-line-width", &focus_width,
+                        "focus-padding", &focus_pad,
+                        NULL);
+
+  window = gtk_widget_get_window (widget);
+
+  _gtk_check_button_get_props (check_button, &indicator_size, &indicator_spacing);
+
+  gtk_widget_get_allocation (widget, &allocation);
+
+  x = indicator_spacing + border_width;
+  y = (allocation.height - indicator_size) / 2;
+
+  child = gtk_bin_get_child (GTK_BIN (check_button));
+  if (!interior_focus || !(child && gtk_widget_get_visible (child)))
+    x += focus_width + focus_pad;      
+
+  if (gtk_toggle_button_get_inconsistent (toggle_button))
+    shadow_type = GTK_SHADOW_ETCHED_IN;
+  else if (gtk_toggle_button_get_active (toggle_button))
+    shadow_type = GTK_SHADOW_IN;
+  else
+    shadow_type = GTK_SHADOW_OUT;
+
+  if (button->priv->activate_timeout || (button->priv->button_down && button->priv->in_button))
+    state_type = GTK_STATE_ACTIVE;
+  else if (button->priv->in_button)
+    state_type = GTK_STATE_PRELIGHT;
+  else if (!gtk_widget_is_sensitive (widget))
+    state_type = GTK_STATE_INSENSITIVE;
+  else
+    state_type = GTK_STATE_NORMAL;
+
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+    x = allocation.width - (indicator_size + x);
+
+  if (gtk_widget_get_state (widget) == GTK_STATE_PRELIGHT)
     {
-      button = GTK_BUTTON (check_button);
-      toggle_button = GTK_TOGGLE_BUTTON (check_button);
-
-      border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-
-      gtk_widget_style_get (widget,
-			    "interior-focus", &interior_focus,
-			    "focus-line-width", &focus_width,
-			    "focus-padding", &focus_pad,
-			    NULL);
-
-      _gtk_check_button_get_props (check_button, &indicator_size, &indicator_spacing);
-
-      x = widget->allocation.x + indicator_spacing + border_width;
-      y = widget->allocation.y + (widget->allocation.height - indicator_size) / 2;
-
-      child = gtk_bin_get_child (GTK_BIN (check_button));
-      if (!interior_focus || !(child && gtk_widget_get_visible (child)))
-	x += focus_width + focus_pad;      
-
-      if (toggle_button->inconsistent)
-	shadow_type = GTK_SHADOW_ETCHED_IN;
-      else if (toggle_button->active)
-	shadow_type = GTK_SHADOW_IN;
-      else
-	shadow_type = GTK_SHADOW_OUT;
-
-      if (button->activate_timeout || (button->button_down && button->in_button))
-	state_type = GTK_STATE_ACTIVE;
-      else if (button->in_button)
-	state_type = GTK_STATE_PRELIGHT;
-      else if (!gtk_widget_is_sensitive (widget))
-	state_type = GTK_STATE_INSENSITIVE;
-      else
-	state_type = GTK_STATE_NORMAL;
-
-      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-	x = widget->allocation.x + widget->allocation.width - (indicator_size + x - widget->allocation.x);
-
-      if (gtk_widget_get_state (widget) == GTK_STATE_PRELIGHT)
-	{
-	  GdkRectangle restrict_area;
-	  GdkRectangle new_area;
-	      
-	  restrict_area.x = widget->allocation.x + border_width;
-	  restrict_area.y = widget->allocation.y + border_width;
-	  restrict_area.width = widget->allocation.width - (2 * border_width);
-	  restrict_area.height = widget->allocation.height - (2 * border_width);
-	  
-	  if (gdk_rectangle_intersect (area, &restrict_area, &new_area))
-	    {
-	      gtk_paint_flat_box (widget->style, widget->window, GTK_STATE_PRELIGHT,
-				  GTK_SHADOW_ETCHED_OUT, 
-				  area, widget, "checkbutton",
-				  new_area.x, new_area.y,
-				  new_area.width, new_area.height);
-	    }
-	}
-
-      gtk_paint_option (widget->style, widget->window,
-			state_type, shadow_type,
-			area, widget, "radiobutton",
-			x, y, indicator_size, indicator_size);
+      gtk_paint_flat_box (style, cr,
+                          GTK_STATE_PRELIGHT,
+                          GTK_SHADOW_ETCHED_OUT, 
+                          widget, "checkbutton",
+                          border_width, border_width,
+                          allocation.width - (2 * border_width),
+                          allocation.height - (2 * border_width));
     }
+
+  gtk_paint_option (style, cr,
+                    state_type, shadow_type,
+                    widget, "radiobutton",
+                    x, y, indicator_size, indicator_size);
 }
